{ mathematical subroutines by TM Chan }
{ 8087 Mode }
unit maths87;
interface

{ Trigonometric Functions in radian}
function tan(x:extended):extended;
function sec(x:extended):extended;
function cosec(x:extended):extended;
function cot(x:extended):extended;
function arcsec(x:extended):extended;
function arccosec(x:extended):extended;
function arccot(x:extended):extended;
function arccos(x:extended):extended;
function arcsin(x:extended):extended;
function arctan2(x,y:extended):extended;

{ Hyperbolic Functions }
function sinh(x:extended):extended;
function cosh(x:extended):extended;
function tanh(x:extended):extended;
function arccosh(x:extended):extended;
function arcsinh(x:extended):extended;
function arctanh(x:extended):extended;

{ Exponential Functions }
function log10(x:extended):extended;    { log x }
function pow(x,y:extended):extended;    { x^y }
function root(x,y:extended):extended;  { x^(1/y) }
function alog10(x:extended):extended;   { 10^x }
function log(y,x:extended):extended;     { log(x) of base y }

{ Fractorial Functions }
function C(n,r:integer):extended;       { nCr }
function P(n,r:integer):extended;       { nPr }
function fra(x:integer):extended;      { x! }

{ Conversion Functions }
function deg(x:extended):extended;      { convert rad to deg }
function rad(x:extended):extended;      { convert deg to rad }
procedure dms(x:extended;var d,m:integer;var s:extended);
                                              { angle to deg,min,sec }
                                              { x is the input }
                                              { d,m,s are the output }
function ang(d,m:integer;s:extended):extended;  { deg,min,sec to angle }
procedure hms(x:extended;var h,m:integer;var s:extended);
                                              { deg,min,sec to hour,min,sec }
                                              { x is the input }
                                              { h,m,s are the output }

implementation
function tan;
begin
     tan:=sin(x)/cos(x)
end;
function sec;
begin
     sec:=1.0/cos(x)
end;
function cosec;
begin
     cosec:=1.0/sin(x)
end;
function cot;
begin
     cot:=cos(x)/sin(x)
end;
function arctan2;
var
   i:extended;
begin
     i:=arctan(abs(y)/abs(x));
     if (x>=0) and (y>=0) then arctan2:=i;
     if (x>=0) and (y<0) then arctan2:=2*pi-i;
     if (x<0) and (y>=0) then arctan2:=pi-i;
     if (x<0) and (y<0) then arctan2:=pi+i
end;

function log10;
begin
     if x=1.0 then log10:=0.0
     else log10:=ln(x)/ln(10.0)
end;
function alog10;
begin
     alog10:=pow(10.0,x)
end;
function arcsec;
begin
     arcsec:=arccos(1/x)
end;
function arccosec;
begin
     arccosec:=arcsin(1/x)
end;
function arccot;
begin
     arccot:=arctan(1/x)
end;
function pow;
var
   i:extended;
begin
     if (x=0.0) and (y<>0.0) then pow:=0.0
     else if y=0.0 then pow:=1.0
          else if (trunc(y)=y) and (x<0) then
               begin
                    i:=exp(y*ln(abs(x)));
                    if odd(trunc(y)) then pow:=-i
                    else pow:=i
               end
               else pow:=exp(y*ln(x));
end;
function root;
begin
     if (x=0.0) and (y<>0.0) then root:=0.0
     else if y=1.0 then root:=x
          else if (trunc(y)=y) and odd(trunc(y)) and (x<0) then root:=-exp(y*ln(1.0/abs(x)))
               else root:=exp(y*ln(1.0/abs(x)));
end;
function fra;
var i:integer;
    z:extended;
begin
     z:=1;
     for i:=1 to x do z:=z*i;
     fra:=z
end;
function P;
var
   i:integer;
   y:extended;
begin
     y:=1.0;
     for i:=n downto n-r+1 do y:=y*i;
     P:=y
end;
function C;
var
   i:integer;
   y:extended;
begin
     y:=1.0;
     for i:=n downto n-r+1 do y:=y*i;
     for i:=2 to r do y:=y/i;
     C:=y
end;
function sinh;
begin
     sinh:=(exp(x)-exp(-x))/2.0
end;
function cosh;
begin
     cosh:=(exp(x)+exp(-x))/2.0
end;
function tanh;
begin
     tanh:=sinh(x)/cosh(x)
end;
function arccosh;
begin
     arccosh:=ln(x+sqrt(sqr(x)-1.0))
end;
function arcsinh;
begin
     arcsinh:=ln(x+sqrt(sqr(x)+1.0))
end;
function arctanh;
begin
     arctanh:=ln((1.0+x)/(1.0-x))/2.0
end;
function deg;
begin
     deg:=frac(x/(2*pi))*360.0
end;
function rad;
begin
     rad:=frac(x/360.0)*2*pi
end;

function arcsin;
begin
     if abs(x)=1.0 then arcsin:=x*pi/2.0
     else arcsin:=arctan(x/sqrt(-x*x+1.0))
end;
function arccos;
begin
     if x=1.0 then arccos:=0.0
     else if x=-1.0 then arccos:=pi
     else arccos:=-arctan(x/sqrt(-x*x+1.0))+pi/2.0
end;
function ang;
begin
     ang:=d+m/60.0+s/3600.0
end;
procedure dms;
begin
     d:=trunc(x);
     m:=trunc(frac(x)*60.0);
     s:=(x-d-m/60.0)*3600.0
end;
procedure hms;
begin
     dms(x/15.0,h,m,s);
end;
function log;
begin
     log:=ln(x)/ln(y)
end;

end.